@@ -8,6 +8,7 @@
 #include <CoreFoundation/CoreFoundation.h>
 #include <ApplicationServices/ApplicationServices.h>
 #include <locale.h>
+#include <sys/utsname.h>
 
 /*
     Typedefs
@@ -20,7 +21,7 @@ typedef void (*Py_SetPathPtr)(const wchar_t*);
 #endif
 typedef int (*PySys_SetObjectPtr)(const char*, PyObject*);
 typedef void (*Py_SetProgramNamePtr)(const char *);
-typedef void (*Py_InitializePtr)(void); 
+typedef void (*Py_InitializePtr)(void);
 typedef int (*PyRun_SimpleFilePtr)(FILE *, const char *);
 typedef void (*Py_FinalizePtr)(void);
 typedef PyObject *(*PySys_GetObjectPtr)(const char *);
@@ -77,7 +78,6 @@ USES(kCFTypeArrayCallBacks);
 USES(CFArrayCreateMutable);
 USES(CFRetain);
 USES(CFRelease);
-USES(CFBundleGetMainBundle);
 USES(CFBundleGetValueForInfoDictionaryKey);
 USES(CFArrayGetCount);
 USES(CFStringCreateWithCString);
@@ -170,7 +170,7 @@ static int bind_objc_Cocoa_ApplicationServices(void) {
 #undef LOOKUP
     return 0;
 }
-    
+
 static int bind_CoreFoundation(void) {
     static Boolean bound = false;
     void *cf_dylib;
@@ -195,7 +195,6 @@ static int bind_CoreFoundation(void) {
     LOOKUP(CFArrayCreateMutable);
     LOOKUP(CFRetain);
     LOOKUP(CFRelease);
-    LOOKUP(CFBundleGetMainBundle);
     LOOKUP(CFBundleGetValueForInfoDictionaryKey);
     LOOKUP(CFArrayGetCount);
     LOOKUP(CFStringCreateWithCString);
@@ -239,7 +238,7 @@ static int bind_CoreFoundation(void) {
     py2app_CFArrayAppendValue(py2app_pool, (const void *)obj), \
     py2app_CFRelease(obj), \
     obj))
-    
+
 #define py2app_CFSTR(s) AUTORELEASE( \
     py2app_CFStringCreateWithCString(NULL, s, kCFStringEncodingUTF8))
 
@@ -262,7 +261,7 @@ static CFTypeRef py2app_getKey(const char *key) {
         key, kCFStringEncodingUTF8);
     if (!cfKey) return NULL;
     rval = py2app_CFBundleGetValueForInfoDictionaryKey(
-        py2app_CFBundleGetMainBundle(),
+        CFBundleGetMainBundle(),
         cfKey);
     py2app_CFRelease(cfKey);
     return rval;
@@ -307,8 +306,8 @@ static int report_error(const char *error) {
     }
     releasePool = ((id(*)(id, SEL))py2app_objc_msgSend)(
 		    ((id(*)(id, SEL))py2app_objc_msgSend)(
-			    py2app_objc_getClass("NSAutoreleasePool"), 
-			    py2app_sel_getUid("alloc")), 
+			    py2app_objc_getClass("NSAutoreleasePool"),
+			    py2app_sel_getUid("alloc")),
 		    py2app_sel_getUid("init"));
     py2app_NSLog(py2app_CFSTR("%@"), py2app_CFSTR(error));
 
@@ -346,7 +345,7 @@ static CFStringRef pyStandardizePath(CFStringRef pyLocation) {
     if (foundRange.location == kCFNotFound || foundRange.length == 0) {
         return NULL;
     }
-    fmwkURL = py2app_CFBundleCopyPrivateFrameworksURL(py2app_CFBundleGetMainBundle());
+    fmwkURL = py2app_CFBundleCopyPrivateFrameworksURL(CFBundleGetMainBundle());
     foundRange.location = foundRange.length;
     foundRange.length = py2app_CFStringGetLength(pyLocation) - foundRange.length;
     subpath = py2app_CFStringCreateWithSubstring(NULL, pyLocation, foundRange);
@@ -401,7 +400,7 @@ static CFStringRef tildeExpand(CFStringRef path) {
     char tmp;
     char *dir = NULL;
 
-    
+
     py2app_CFStringGetCString(path, buf, sizeof(buf), kCFStringEncodingUTF8);
 
     int i;
@@ -450,7 +449,7 @@ static void py2app_setPythonPath(void) {
     CFDictionaryRef options;
 
     paths = py2app_CFArrayCreateMutable(NULL, 0, py2app_kCFTypeArrayCallBacks);
-    resDir = py2app_CFBundleCopyResourcesDirectoryURL(py2app_CFBundleGetMainBundle());
+    resDir = py2app_CFBundleCopyResourcesDirectoryURL(CFBundleGetMainBundle());
 
     resPath = pathFromURL(resDir);
     py2app_CFArrayAppendValue(paths, resPath);
@@ -529,7 +528,7 @@ static void py2app_setPythonPath(void) {
 static void setResourcePath(void) {
     CFURLRef resDir;
     CFStringRef resPath;
-    resDir = py2app_CFBundleCopyResourcesDirectoryURL(py2app_CFBundleGetMainBundle());
+    resDir = py2app_CFBundleCopyResourcesDirectoryURL(CFBundleGetMainBundle());
     resPath = pathFromURL(resDir);
     py2app_CFRelease(resDir);
     setcfenv("RESOURCEPATH", resPath);
@@ -567,7 +566,7 @@ static CFStringRef getMainScript(void) {
     e_pyo = py2app_CFSTR("pyo");
 
     cnt = py2app_CFArrayGetCount(possibleMains);
-    bndl = py2app_CFBundleGetMainBundle();
+    bndl = CFBundleGetMainBundle();
     path = NULL;
     for (i = 0; i < cnt; i++) {
         CFStringRef base;
@@ -613,7 +612,7 @@ static CFStringRef getPythonInterpreter(CFStringRef pyLocation) {
 
     auxName = py2app_getKey("PyExecutableName");
     if (!auxName) auxName = py2app_CFSTR("python");
-    bndl = py2app_CFBundleGetMainBundle();
+    bndl = CFBundleGetMainBundle();
     auxURL = py2app_CFBundleCopyAuxiliaryExecutableURL(bndl, auxName);
     if (auxURL) {
         path = pathFromURL(auxURL);
@@ -643,7 +642,7 @@ static CFStringRef getErrorScript(void) {
     py2app_CFArrayAppendValue(errorScripts, py2app_CFSTR("__error__.sh"));
 
     cnt = py2app_CFArrayGetCount(errorScripts);
-    bndl = py2app_CFBundleGetMainBundle();
+    bndl = CFBundleGetMainBundle();
     path = NULL;
     for (i = 0; i < cnt; i++) {
         CFStringRef base;
@@ -658,7 +657,7 @@ static CFStringRef getErrorScript(void) {
     }
     py2app_CFRelease(errorScripts);
     return path;
- 
+
 }
 
 static CFMutableArrayRef get_trimmed_lines(CFStringRef output) {
@@ -710,13 +709,13 @@ static int report_script_error(const char *msg) {
         CFMutableArrayRef argv;
         releasePool = ((id(*)(id, SEL))py2app_objc_msgSend)(
 		    ((id(*)(id, SEL))py2app_objc_msgSend)(
-			    py2app_objc_getClass("NSAutoreleasePool"), 
-			    py2app_sel_getUid("alloc")), 
+			    py2app_objc_getClass("NSAutoreleasePool"),
+			    py2app_sel_getUid("alloc")),
 		    py2app_sel_getUid("init"));
         task = ((id(*)(id, SEL))py2app_objc_msgSend)(
 		    ((id(*)(id, SEL))py2app_objc_msgSend)(
-			    py2app_objc_getClass("NSTask"), 
-			    py2app_sel_getUid("alloc")), 
+			    py2app_objc_getClass("NSTask"),
+			    py2app_sel_getUid("alloc")),
 		    py2app_sel_getUid("init"));
         stdoutPipe = ((id(*)(id, SEL))py2app_objc_msgSend)(py2app_objc_getClass("NSPipe"), py2app_sel_getUid("pipe"));
         ((void(*)(id, SEL, id))py2app_objc_msgSend)(task, py2app_sel_getUid("setLaunchPath:"), py2app_CFSTR("/bin/sh"));
@@ -798,7 +797,7 @@ static int report_script_error(const char *msg) {
             if (!buttonString) buttonString = py2app_CFSTR(ERR_DEFAULTURLTITLE);
         }
         py2app_CFRelease(buttonArr);
-        
+
     }
     if (lineCount <= 0 || errBinding) {
         py2app_CFRelease(lines);
@@ -807,8 +806,8 @@ static int report_script_error(const char *msg) {
 
     releasePool = ((id(*)(id, SEL))py2app_objc_msgSend)(
 		    ((id(*)(id, SEL))py2app_objc_msgSend)(
-			    py2app_objc_getClass("NSAutoreleasePool"), 
-			    py2app_sel_getUid("alloc")), 
+			    py2app_objc_getClass("NSAutoreleasePool"),
+			    py2app_sel_getUid("alloc")),
 		    py2app_sel_getUid("init"));
 
     title = py2app_CFArrayGetValueAtIndex(lines, 0);
@@ -887,6 +886,14 @@ static int py2app_main(int argc, char * const *argv, char * const *envp) {
         unsetenv("PYTHONMALLOCSTATS");
     }
 
+    /* Ensure that the interpreter won't try to write bytecode files
+     * Two reasons:
+     * - Apps are often read-only for users
+     * - Writing byte-code will be blocked by the sandbox
+     *   when running a sandboxed application.
+     */
+    setenv("PYTHONDONTWRITEBYTECODE", "1", 1);
+
 
     if (!py2app_getApplicationName()) return report_error(ERR_NONAME);
     pyLocations = (CFArrayRef)py2app_getKey("PyRuntimeLocations");
@@ -939,11 +946,11 @@ static int py2app_main(int argc, char * const *argv, char * const *envp) {
 	    NAME ## Ptr py2app_ ## NAME = (NAME ## Ptr)dlsym(py_dylib, #NAME); \
 	    if (!py2app_ ## NAME) { \
 		return report_linkEdit_error(); \
-	    } 
+	    }
 
 #define OPT_LOOKUP(NAME) \
-	    NAME ## Ptr py2app_ ## NAME = (NAME ## Ptr)dlsym(py_dylib, #NAME); 
-    
+	    NAME ## Ptr py2app_ ## NAME = (NAME ## Ptr)dlsym(py_dylib, #NAME);
+
     LOOKUP(Py_SetProgramName);
     LOOKUP(Py_Initialize);
     LOOKUP(PyRun_SimpleFile);
@@ -963,37 +970,36 @@ static int py2app_main(int argc, char * const *argv, char * const *envp) {
 
 #undef OPT_LOOKUP
 #undef LOOKUP
-	
-    /*
-     * When apps are started from the Finder (or anywhere
-     * except from the terminal), the LANG and LC_* variables
-     * aren't set in the environment. This confuses Py_Initialize
-     * when it tries to import the codec for UTF-8, 
-     * therefore explicitly set the locale. 
-     *
-     * Also set the LC_CTYPE environment variable because Py_Initialize
-     * resets the locale information using the environment :-(
-     */
-    curlocale = setlocale(LC_ALL, NULL);
-    if (curlocale != NULL) {
-      curlocale = strdup(curlocale);
-      if (curlocale == NULL) {
-        (void)report_error("cannot save locale information");
-	return -1;
-      }
-    }
-    setlocale(LC_ALL, "en_US.UTF-8");
-
-    curenv = getenv("LC_CTYPE");
-    if (curenv) {
-	    curenv = strdup(curenv);
-    }
-    setenv("LC_CTYPE", "en_US.UTF-8", 1);
 
     if (isPy3K) {
-    	wchar_t w_pythonInterpreter[PATH_MAX+1];
-    	mbstowcs(w_pythonInterpreter, c_pythonInterpreter, PATH_MAX+1);
-    	py2app_Py_SetProgramName((char*)w_pythonInterpreter);
+	    /*
+	     * When apps are started from the Finder (or anywhere
+	     * except from the terminal), the LANG and LC_* variables
+	     * aren't set in the environment. This confuses Py_Initialize
+	     * when it tries to import the codec for UTF-8,
+	     * therefore explicitly set the locale.
+	     *
+	     * Also set the LC_CTYPE environment variable because Py_Initialize
+	     * resets the locale information using the environment :-(
+	     */
+	    curlocale = setlocale(LC_ALL, NULL);
+	    if (curlocale != NULL) {
+	      curlocale = strdup(curlocale);
+	      if (curlocale == NULL) {
+		(void)report_error("cannot save locale information");
+		return -1;
+	      }
+	    }
+	    setlocale(LC_ALL, "en_US.UTF-8");
+
+	    curenv = getenv("LC_CTYPE");
+	    if (curenv == NULL) {
+		setenv("LC_CTYPE", "en_US.UTF-8", 1);
+	    }
+
+	    wchar_t w_pythonInterpreter[PATH_MAX+1];
+    	    mbstowcs(w_pythonInterpreter, c_pythonInterpreter, PATH_MAX+1);
+    	    py2app_Py_SetProgramName((char*)w_pythonInterpreter);
 
 
     } else {
@@ -1005,14 +1011,13 @@ static int py2app_main(int argc, char * const *argv, char * const *envp) {
     /*
      * Reset the environment and locale information
      */
-    setlocale(LC_CTYPE, curlocale);
-    free(curlocale);
+    if (isPy3K) {
+	    if (curenv == NULL) {
+		unsetenv("LC_CTYPE");
+	    }
 
-    if (curenv) {
-	    setenv("LC_CTYPE", curenv, 1);
-	    free(curenv);
-    } else {
-	    unsetenv("LC_CTYPE");
+	    setlocale(LC_CTYPE, curlocale);
+	    free(curlocale);
     }
 
     py2app_CFStringGetCString(
@@ -1042,7 +1047,7 @@ static int py2app_main(int argc, char * const *argv, char * const *envp) {
     mainScriptFile = fopen(c_mainScript, "r");
     rval = py2app_PyRun_SimpleFile(mainScriptFile, c_mainScript);
     fclose(mainScriptFile);
-    
+
     if (rval) {
         rval = report_script_error(ERR_PYTHONEXCEPTION);
     }
@@ -1052,14 +1057,166 @@ static int py2app_main(int argc, char * const *argv, char * const *envp) {
     return rval;
 }
 
-int main(int argc, char * const *argv, char * const *envp)
+void
+setup_asl(const char* appname)
+{
+	/*
+	 * On Mac OS X 10.8 or later the contents of stdout and stderr
+	 * do not end up in de Console.app log (neither in de default
+	 * view or some other view).
+	 *
+	 * This function detects if "asl_log_descriptor" is available
+	 * (introduced in 10.8), and if it is configures ASL to redirect
+	 * all writes to stdout/stderr to the ASL in such a way that
+	 * log lines end up in the default view of Console.app.
+	 */
+
+	/* Function definitions, don't use the ASL header because that
+	 * doesn't contain all definitions when building the binaries
+	 * with PPC support.
+	 */
+	typedef void* aslclient;
+	typedef void* aslmsg;
+
+	aslclient (*do_asl_open)(const char*, const char*, int);
+	int (*do_asl_add_log_file)(aslclient, int);
+	int (*do_asl_log_descriptor)(aslclient, aslmsg, int, int, uint32_t);
+	int (*do_asl_set)(aslmsg, const char*, const char*);
+	aslmsg (*do_asl_new)(int);
+
+	/*
+	 * Try to resolve the ASL function's we're using.
+	 */
+        void* sys_dylib;
+        sys_dylib = dlopen("/usr/lib/libSystem.dylib", RTLD_LAZY);
+
+	do_asl_open = dlsym(sys_dylib, "asl_open");
+	do_asl_add_log_file = dlsym(sys_dylib, "asl_add_log_file");
+	do_asl_log_descriptor = dlsym(sys_dylib, "asl_log_descriptor");
+	do_asl_set = dlsym(sys_dylib, "asl_set");
+	do_asl_new = dlsym(sys_dylib, "asl_new");
+
+	if (do_asl_log_descriptor == NULL) {
+		/* asl_log_descriptor is not available: running on OSX 10.7
+		 * or earlier, which means we can't set up ASL and don't have to
+		 */
+		return;
+	}
+
+	aslclient cl;
+	aslmsg msg;
+	int fd2;
+	char uidbuf[64];
+
+	cl = do_asl_open(appname, "com.apple.console", 2 /* ASL_OPT_NO_DELAY */);
+	if (cl == NULL) {
+		return;
+	}
+
+	/* Tell ASL to write all log lines to STDERR as well, use a dup-ed file
+	 * descriptor because we'll later replace the original one by a file descriptor
+	 * from ASL.
+	 */
+	fd2 = dup(2);
+	do_asl_add_log_file(cl, fd2);
+
+	/*
+	 * Create an ASL template message for the STDOUT/STDERR redirection.
+	 * All keys are required to get log messages into the default view of Console.app
+	 * for normal users.
+	 */
+	snprintf(uidbuf, sizeof(uidbuf), "%d", getuid());
+	msg = do_asl_new(0 /* ASL_TYPE_MSG */);
+	do_asl_set(msg, "Facility" /* ASL_KEY_FACILITY */, "com.apple.console");
+	do_asl_set(msg, "Level" /* ASL_KEY_LEVEL */, "Notice" /* ASL_STRING_NOTICE */);
+	do_asl_set(msg, "ReadUID" /* ASL_KEY_READ_UID */, uidbuf /* or "-1" for "All users" */);
+
+	/*
+	 * Finally: redirect the STDOUT/STDERR file descriptors to ASL.
+	 */
+	do_asl_log_descriptor(cl, msg, 4 /* ASL_LEVEL_NOTICE */, 1, 2 /* ASL_LOG_DESCRIPTOR_WRITE */);
+	do_asl_log_descriptor(cl, msg, 4 /* ASL_LEVEL_NOTICE */, 2, 2 /* ASL_LOG_DESCRIPTOR_WRITE */);
+}
+
+#ifndef PY2APP_SECONDARY
+static int
+have_psn_arg(int argc, char* const * argv)
+{
+	int i;
+
+	for (i = 0; i < argc; i++) {
+		if (strncmp(argv[i], "-psn_", 5) == 0) {
+			if (isdigit(argv[i][5])) {
+				return 1;
+			}
+		}
+	}
+	return 0;
+}
+#endif /* !PY2APP_SECONDARY */
+
+
+int
+main(int argc, char * const *argv, char * const *envp)
 {
     int rval;
+
+#ifndef PY2APP_SECONDARY
+    /* Running as a GUI app started by launch
+     * services, try to redirect stdout/stderr
+     * to ASL.
+     *
+     * NOTE: Detecting application bundles on OSX 10.9
+     * is annoyingly hard, the devnull trick is the least
+     * worst option I've found yet.
+     */
+    struct stat st;
+    struct utsname uts;
+    int is_app_bundle = 1;
+
+    if (uname(&uts) != -1) {
+        if (strcmp(uts.release, "13.") <= 0) {
+	    /* OSX 10.8 or earlier */
+            if (!have_psn_arg(argc, argv)) {
+                is_app_bundle = 0;
+	    }
+	} else {
+	    /* OSX 10.9 or later */
+            if (fstat(1, &st) != -1) {
+	        if (!S_ISCHR(st.st_mode) ||
+	            major(st.st_dev) != 3 ||
+     	            minor(st.st_dev) != 2) {
+
+    	    	        /* We appear to be launched from the
+			 * command-line after all.
+		         */
+		        is_app_bundle = st.st_dev;
+	        }
+            }
+        }
+    }
+
+
+    if (is_app_bundle) {
+        const char *bname;
+        setenv("_PY2APP_LAUNCHED_", "1", 1);
+
+        bname = strrchr(argv[0], '/');
+        if (bname == NULL) {
+	    bname = argv[0];
+        } else {
+	    bname++;
+        }
+
+        setup_asl(bname);
+    }
+#endif /* !PY2APP_SECONDARY */
+
     if (bind_CoreFoundation()) {
         fprintf(stderr, "CoreFoundation not found or functions missing\n");
         return -1;
     }
-    if (!py2app_CFBundleGetMainBundle()) {
+    if (!CFBundleGetMainBundle()) {
         fprintf(stderr, "Not bundled, exiting\n");
         return -1;
     }
