@@ -21,7 +21,7 @@ gPreBuildVariants = [
         'name': 'main-x86_64',
         'target': '10.5',
         'cflags': '-isysroot / -arch x86_64',
-        'cc': 'gcc-4.2',
+        'cc': 'clang',
     },
     {
         'name': 'main-fat3',
@@ -33,14 +33,14 @@ gPreBuildVariants = [
         'name': 'main-intel',
         'target': '10.5',
         'cflags': '-isysroot / -arch i386 -arch x86_64',
-        'cc': 'gcc-4.2',
+        'cc': 'clang',
     },
     {
         'name': 'main-i386',
         'target': '10.3',
         #'cflags': '-isysroot @@XCODE_ROOT@@/SDKs/MacOSX10.4u.sdk -arch i386',
-        'cflags': '-arch i386',
-        'cc': 'gcc-4.2',
+        'cflags': '-arch i386 -isysroot /',
+        'cc': 'clang',
     },
     {
         'name': 'main-ppc',
@@ -85,13 +85,11 @@ def main(all=False, arch=None):
         arch = distutils.util.get_platform().split('-')[-1]
         if sys.prefix.startswith('/System') and \
                 sys.version_info[:2] == (2,5):
-                    arch = "fat"
+            arch = "fat"
 
     name = 'main-' + arch
-
     root = None
 
-
     if all:
         for entry in gPreBuildVariants:
             if (not all) and entry['name'] != name: continue
