@@ -14,7 +14,7 @@ setup(
         resources = "data3/source.c",
     )),
     data_files = [
-        ( 'sub1', [ 'data1/file1.txt', 'data1/file2.txt' ]),
-        'data2' 
+        ( 'sub1', [ 'data1/file1.txt', 'data1/file2.txt', 'data1/file3.sh' ]),
+        'data2'
     ]
 )
